SystemOrganization addCategory: #'Control-Core'!
SystemOrganization addCategory: #'Control-Exceptions'!

Error subclass: #MissingResetException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Control-Exceptions'!

Notification subclass: #DelimitedControlException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Control-Exceptions'!

!DelimitedControlException commentStamp: 'fbs 3/31/2011 17:39' prior: 0!
I mark a context so that a shift further up the stack will unwind only to me. In other words, use me to delimit a continuation.!

Object subclass: #PartialContinuation
	instanceVariableNames: 'values'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Control-Core'!

!PartialContinuation commentStamp: 'fbs 4/12/2011 18:58' prior: 0!
I am a slice of a call stack, that is to say, some sequence of ContextParts. Use me like a Block by calling my #value or #value: methods.

My serialisation/deserialisation code comes straight from Seaside's WAPartialContinuation.!

!PartialContinuation class methodsFor: 'instance creation' stamp: 'fbs 4/12/2011 19:11'!
from: topContext downTo: bottomContext
	^ self new from: topContext downTo: bottomContext.! !

!PartialContinuation methodsFor: 'private' stamp: 'fbs 4/12/2011 21:16'!
deserialiseFrom: anObject
	| stream context |
	stream := anObject readStream.
	"We don't need to explicitly connect the contexts, because one of the instvars is sender."
	[stream atEnd] whileFalse:
		[context := stream next.
		1 to: context class instSize do: [ :index | context instVarAt: index put: (stream next) ].
		1 to: context size do: [ :index | context at: index put: (stream next) ] ].
	^ context! !

!PartialContinuation methodsFor: 'initialize-release' stamp: 'fbs 4/13/2011 07:32'!
from: topContext downTo: bottomContext
	values := self serialiseFrom: topContext to: bottomContext.! !

!PartialContinuation methodsFor: 'accessing' stamp: 'fbs 4/14/2011 15:41'!
numArgs
	^ 1! !

!PartialContinuation methodsFor: 'private' stamp: 'fbs 4/14/2011 15:20'!
serialiseFrom: aContext to: lowerContext
	| c stream |
	stream := WriteStream on: (OrderedCollection new: 100).
"	Transcript showln: '8<-----'."
	c := aContext.
	[c isNil or: [c == lowerContext]] whileFalse:
		["Transcript showln: c printString."
		stream nextPut: c.
		1 to: c class instSize do: [ :index | stream nextPut: (c instVarAt: index) ].
		1 to: c size do: [ :index | stream nextPut: (c at: index) ].
		c := c sender.].
	c ifNil: [MissingResetException signal: lowerContext printString , ' not found while capturing continuation'].
	
	^ stream contents.! !

!PartialContinuation methodsFor: 'accessing' stamp: 'fbs 4/14/2011 07:35'!
stack
	"A debugging aid. Return a Collection containing my contexts."
	| result stream context |
	result := OrderedCollection new.
	stream := values readStream.

	[stream atEnd] whileFalse:
		[context := stream next.
		1 to: context class instSize do: [ :index | stream next ].
		1 to: context size do: [ :index | stream next ].
		result addLast: context].
	
	^ result.! !

!PartialContinuation methodsFor: 'evaluating' stamp: 'fbs 4/12/2011 18:59'!
value
	"Splice my contents onto the stack, and execute me."
	self value: nil.! !

!PartialContinuation methodsFor: 'evaluating' stamp: 'fbs 4/12/2011 19:10'!
value: anObject
	"Splice my contents onto the stack, and execute me."
	(self deserialiseFrom: values) swapSender: thisContext sender.
	thisContext swapSender: values first.
	^ anObject.! !
